{-# LANGUAGE FlexibleInstances #-}

module Values (
  Values,
  Values2D,
  Values3D,
  dims,
  empty3d,
  idx,
  zoom_shape )
where

import Data.Array.Repa (
  Array, U,
  Z( Z ),
  (:.)( (:.) ),
  DIM2,
  DIM3,
  extent,
  fromListUnboxed,
  unsafeIndex )

import ScaleFactor ( ScaleFactor )


type Values sh = Array U sh Double
type Values2D  = Values DIM2
type Values3D  = Values DIM3


empty3d :: Values3D
empty3d = Data.Array.Repa.fromListUnboxed (Z :. 0 :. 0 :. 0) []


dims :: Values3D -> (Int, Int, Int)
dims v3d =
    let (Z :. x :. y :. z) = extent v3d
    in
      (x,y,z)


idx :: Values3D -> Int -> Int -> Int -> Double
idx v3d i j k =
    unsafeIndex v3d shape
    where
      shape :: DIM3
      shape = (Z :. i :. j :. k)


zoom_shape :: ScaleFactor -> DIM3 -> DIM3
zoom_shape (sfx, sfy, sfz) sh =
    let (Z :. x :. y :. z) = sh
        x' = x * sfx
        y' = y * sfy
        z' = z * sfz
    in
      (Z :. x' :. y' :. z')
